import os

def get_size_file(file_path):
    return os.path.getsize(file_path)

def format_size(size):
    if size < 1024:
        return str(size) + "B"
    elif size < 1024 * 1024:
        return str(round(size / 1024, 2)) + "KB"
    else:
        return str(round(size / 1024 / 1024, 2)) + "MB"

def get_format_size(file_path):
    return format_size(get_size_file(file_path))



import os
from pathlib import Path

def copiar_exe(path_exe, path_destino):
    total_size = os.path.getsize(path_exe)
    total_copiado = 0
    with open(path_exe, 'rb') as src_file:
        with open(path_destino, 'wb') as dest_file:
            while chunk := src_file.read(8192):
                    dest_file.write(chunk)
                    total_copiado += len(chunk)
                    yield total_copiado * 100 / total_size

def agregar_al_Path(path_instalacion:Path):
    os.system(f'SETX PATH %PATH%;"{path_instalacion}"')


from typing import Callable, Any, Union
from ..components import Controles
import flet as ft

class Pagina:
    def __init__(self, func:Callable[[ft.Page, Controles, Any], tuple], titulo:str="") -> None:
        self.pag = func
        self.titulo = titulo

class Paginas:
    def __init__(self, pags:dict[str, Union[Callable, Pagina]]={}):
        self.paginas:dict = {}
        for k, pag in pags.items():
            self.agregar(k, pag)
    
    def get_titulo_pag(self, id) -> Union[str, None]:
        pag = self.paginas.get(id, None)
        if pag:
            return pag[1] if isinstance(pag, (tuple, list)) and len(pag) > 1 else ""
        return None

    def agregar(self, ruta:str, pag:Union[Callable[[ft.Page, Controles, Any], tuple], Pagina]):
        pagina = pag if isinstance(pag, Pagina) else Pagina(pag, "")
        self.paginas[ruta] = pagina
    
    def get(self, ruta:str) -> Union[Pagina, None]:
        return self.paginas.get(ruta, None)
    
    def __len__(self):
        return len(self.paginas)




import flet as ft

class Modal_Cerrar:
    def __init__(self,  titulo, contenido, txt_accept, txt_cancel):
        self.__page = None
        self.titulo = titulo
        self.contenido = contenido
        self.titulo = titulo
        self.btn_accept = txt_accept
        self.btn_cancel = txt_cancel
    
    def crear(self, page) -> ft.AlertDialog:
        self.__page = page
        def yes_click(e):
            self.__page.window_destroy()

        def no_click(e):
            confirm_dialog.open = False
            self.__page.update()
            
        confirm_dialog = ft.AlertDialog(
            modal=True,
            title=ft.Text(self.titulo),
            content=ft.Text(self.contenido),
            actions=[
                ft.ElevatedButton(self.btn_accept, on_click=yes_click),
                ft.OutlinedButton(self.btn_cancel, on_click=no_click),
            ],
            actions_alignment=ft.MainAxisAlignment.END,
        )
        return confirm_dialog


import flet as ft
from pathlib import Path
from typing import Callable

from .helpers.install_helpers import copiar_exe, agregar_al_Path
from .components import Modal_Cerrar, Controles
from .pages import Paginas
from .vars import InstallerVars

class Regalo():
    def __init__(self, nombre_regalo:str, path_instalador_exe:str, paginas:Paginas, vars_config:dict={}, args_modal_cerrar:dict=None, func_config:Callable=None, mostrar_siempre_appbar:bool=False):
        self.pages:Paginas = paginas
        self.__page:ft.Page = None

        self.path_instalador_exe:Path = Path(path_instalador_exe)
        if not self.path_instalador_exe.exists():
            raise FileNotFoundError(f"El archivo {self.path_instalador_exe} no existe")
        
        self.nombre_regalo = nombre_regalo

        self.kwargs_app_flet = None
        self.mostrar_siempre_appbar = mostrar_siempre_appbar
        self.func_config = func_config
        self.variables_instalador = InstallerVars(vars_config)
        self.variables_instalador["path_instalador"] = path_instalador_exe

        # Componentes de dialog para cerrar
        self.args_modal_cerrar = args_modal_cerrar or {
            "titulo": "En serio?",
            "contenido": "Quieres cancelar la instalación?",
            "txt_accept": "Yep",
            "txt_cancel": "No ._.",
        }
        self.confirm_dialog = Modal_Cerrar(**self.args_modal_cerrar)

        self.controles_inst:Controles = Controles()

    def handler_cerrar_instalador(self) -> ft.AlertDialog:
        return self.confirm_dialog.crear(self.__page)
    
    def abrir(self, main, **kwargs):
        self.kwargs_app_flet = kwargs
        self.main = main
        ft.app(target=self.app, view=ft.AppView.FLET_APP)
    
    def instalar_regalo(self, pb:ft.ProgressBar, path_instalacion):
        path_instalacion = Path(path_instalacion)
        if path_instalacion.name.lower() != self.nombre_regalo.lower():
            path_instalacion = path_instalacion / self.nombre_regalo
        
        if path_instalacion.exists():
            path_instalacion.rmdir()
        path_instalacion.mkdir(parents=True, exist_ok=True)  
        
        ext = self.path_instalador_exe.suffix
        filename = f"{self.path_instalador_exe.stem}{ext}"

        pb.value = 0
        for porcentaje in copiar_exe(self.path_instalador_exe, path_instalacion / filename):
            pb.value = porcentaje

        if self.variables_instalador.get("agregar_path"):
            agregar_al_Path(path_instalacion)

    def __route_change(self, event:ft.RouteChangeEvent):
        ruta = event.route

        pagina = self.pages.get(ruta)
        if not pagina: return
        self.render(ruta, pagina)

    def __llamar_view(self, func):
        return func(self.__page, self.controles_inst, self.variables_instalador)

    def __view_pop(self, event):
        self.__page.views.pop()
        top_view = self.__page.views.pop()
        self.__page.go(top_view.route)

    def render(self, ruta, pagina):
        self.controles_inst.appbar.title = pagina.titulo
        contenido = self.__llamar_view(pagina.pag)
        appBar = self.controles_inst.appbar
        appBar.title = pagina.titulo if appBar.title == "" and pagina.titulo else appBar.title

        render = [
            appBar,
            ft.Container(content=ft.Column(contenido), expand=True),
            self.controles_inst.render(alignment=ft.MainAxisAlignment.END)
        ]
        
        if not self.mostrar_siempre_appbar and ruta == "/": render.pop(0)

        if contenido:
            self.__page.views.append(ft.View(ruta, render))
        self.__page.update()

    def inicial_config(self, page):    
        for k,v in self.kwargs_app_flet.items():
            setattr(page, k, v)
            
        page.on_route_change = self.__route_change
        page.on_view_pop = self.__view_pop
        
        confirm_dialog = self.handler_cerrar_instalador()

        def window_event(e):
            if e.data == "close" or not e.data:
                page.dialog = confirm_dialog
                confirm_dialog.open = True
                page.update()

        self.controles_inst.agregar_boton("next", "Next")
        self.controles_inst.agregar_boton("cancel", "Cancel", on_click=window_event)
        self.controles_inst.agregar_pb("pb")

    def app(self, page: ft.Page):
        self.pages.agregar("/", self.main)
        self.__page = page
        self.inicial_config(self.__page)
        if self.func_config: self.func_config(self.__page, self.controles_inst, self.variables_instalador)

        page.views.clear()
        self.render("/", self.pages.get("/"))
        


from typing import Any

class InstallerVars(dict):
    def __init__(self, *args, **kwargs):
        super(InstallerVars, self).__init__(*args, **kwargs)
    
    def __getattr__(self, key: str) -> Any:
        return self.get(key, None)


from setuptools import setup, find_packages

with open('README.md', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='Regalo',
    version='0.1',
    license='MIT',
    description='Creador de instaladores para windows',
    long_description_content_type="text/markdown",
    long_description=readme,
    author='Nakato',
    author_email='christianvelasces@gmail.com',
    url='https://github.com/nakato156/Regalo',
    keywords=['windows', 'installer', 'exe', "executable", "package"],
    packages=find_packages(),
)


import flet as ft
from typing import Tuple, Any

class Controles:
    def __init__(self) -> None:
        valType = Tuple[ft.Ref[ft.TextField], ft.ElevatedButton, tuple]
        
        self.botones: dict[str, valType] = {}
        self.controles: dict[str, Any] = {}

        self.appbarRef:ft.Ref[ft.AppBar] = ft.Ref[ft.AppBar]()
        self.appbar:ft.AppBar = ft.AppBar(ref=self.appbarRef)

    def __getitem__(self, item):
        if item in self.botones.keys():
            return self.botones[item][1]
        elif item in self.controles.keys():
            return self.controles[item][1]

    def get_btn(self, id: str) -> ft.Ref[ft.ElevatedButton]:
        return self.botones[id][1]
    
    def get_control(self, id: str) -> ft.Ref[Any]:
        return self[id]
    
    def __getattr__(self, name: str) -> ft.Ref[ft.ElevatedButton]:
        if name in self.botones.keys():
            return self.botones[name][0]
        elif name in self.controles.keys():
            return self.controles[name][0]
        return super().__getattribute__(name)

    def agregar_boton(self, id, texto, **kwargs:dict):
        ref_btn = ft.Ref[ft.ElevatedButton]()
        btn = ft.ElevatedButton(texto, ref=ref_btn, **kwargs)
        self.botones[id] = (ref_btn, btn, tuple(kwargs.items()))
        return btn

    def agregar_pb(self, id, **kwargs):
        ref_pb = ft.Ref[ft.ProgressBar]()
        pb = ft.ProgressBar(ref=ref_pb )
        self.controles[id] = (ref_pb, pb)
        return pb
    
    def eliminar_control(self, id):
        if id in self.botones.keys(): del self.botones[id]
        if id in self.controles.keys(): del self.controles[id]

    def reset(self):
        for ref, btn, args in self.botones.values():
            for name, val in args:
                setattr(btn, name, val)
            ref.current.update()

    def render(self, **kwargs) -> ft.Row:
        return ft.Row([btn[1] for btn in self.botones.values()], **kwargs)